<?php
namespace Joson\Rest\Command;

/*
 * This file is part of the Joson.Rest package.
 */

use TYPO3\Flow\Annotations as Flow;

/**
 * @Flow\Scope("singleton")
 */
class UserCommandController extends \TYPO3\Flow\Cli\CommandController
{
   const user_role = 'Joson.Test:Admin';
   const authentication_provider = 'DefaultProvider';
   const password_hashing_strategy = 'default';

    /**
    * @Flow\Inject
    * @var \TYPO3\Flow\Security\AccountFactory
    */
    protected $accountFactory;

    /**
     * @var \TYPO3\Flow\Security\AccountRepository
     * @Flow\Inject
     */
    protected $accountRepository;

    /**
    * @Flow\Inject
    * @var \Joson\Rest\Domain\Repository\RealUsersRepository
    */
    protected $realUsersRepository;

    /**
    * @Flow\Inject
    * @var \TYPO3\Flow\Security\Cryptography\HashService
    */
    protected $hashService;

    /**
     * This command is used for adding authenticated user in the current system.
     *
     * @param string $username
     * @param string $password
     * @Flow\Validate(argumentName="username", type="NotEmpty")
     * @Flow\Validate(argumentName="password", type="NotEmpty")
     * @return void
     */
    public function createCommand($username, $password )
    {

      $this->outputLine('Trying to create a user with the username "%s", password "%s".', [$username, $password]);

      $duplicateUser = $this->realUsersRepository->findOneByUsername($username);
      if( empty($duplicateUser) === false ) {
        $this->outputLine("Duplicate user.");
      } else {
        $account = $this->accountFactory->createAccountWithPassword($username, $password, [self::user_role], self::authentication_provider);
        $token = $this->hashService->generateHmac($username);
        $realUser = new \Joson\Rest\Domain\Model\RealUsers();
        $realUser->setUsername($username);
        $realUser->setPassword($password);
        $realUser->setUserAccount($account);
        $realUser->setToken($token);
        $this->accountRepository->add($account);
        $this->realUsersRepository->add($realUser);
        $this->outputLine('Added the user "%s" to the system.', [$username]);
        $this->outputLine('Authenticated Token is "%s".', [$token]);
      }

    }

    /**
     * @param  string $username
     */
    public function removeCommand($username, $authentication_provider=NULL)
    {
        $this->outputLine('Trying to remove the user "%s" from system.', [$username]);
        $authentication_provider = ($authentication_provider!=NULL) ? $authentication_provider : self::authentication_provider;
        $account = $this->getAccountByIdentifier($username, $authentication_provider);
        if ($account != NULL) {
            $this->accountRepository->remove($account);
            $realUser = $this->realUsersRepository->findOneByUsername($username);
            if ($realUser!=NULL) $this->realUsersRepository->remove($realUser);
            $this->outputLine('Removed the user "%s" from system.', [$username]);
        } else {
            $this->outputLine('Cant find the user "%s" in the system.', [$username]);
        }
    }

   private function getAccountByIdentifier($identifier, $authentication_provider)
   {
       return $this->accountRepository->findByAccountIdentifierAndAuthenticationProviderName($identifier, $authentication_provider);
   }

}
