<?php
namespace Joson\Rest\Controller;

/*
 * This file is part of the Joson.Rest package.
 */

use TYPO3\Flow\Annotations as Flow;

class StandardController extends \TYPO3\Flow\Mvc\Controller\ActionController {

	/**
	 * @Flow\Inject
	 * @var \Joson\Rest\Domain\Repository\UserRepository
	 */
	protected $userRepository;

	/**
	* @Flow\Inject
	* @var \Joson\Rest\Domain\Repository\RealUsersRepository
	*/
	protected $realUsersRepository;

	/**
	 * Adding user (PUT HTTP method)
	 * @return void
	 */
	public function indexAction() {
		$flag1    = 0;
		$name = "";
		$age  = "";
		try {
			$checkToken = $this->hasAuthenticatedUser();
			if( $checkToken === true ) {
				if ($this->request->hasArgument("name") == false) {
					$this->response->setStatus(400, "Bad Request");
					$flag1   = 1;
				} else {
					$name =$this->request->getArgument("name");
					if( empty($name) === true ) {
						$this->response->setStatus(400, "Bad Request");
						$flag1   = 1;
					} else {
						if ($this->request->hasArgument("age") == false) {
							$this->response->setStatus(400, "Bad Request");
							$flag1   = 1;
						} else {
							$age = $this->request->getArgument("age");
							if( empty($age) === true ) {
								$this->response->setStatus(400, "Bad Request");
								$flag1   = 1;
							}
						}
					}

				}
				if ($flag1 == 0) {
					$user = new \Joson\Rest\Domain\Model\User();
					$user->setName($name);
					$user->setAge($age);
					$this->userRepository->add($user);
					$this->response->setStatus(201, "Created");
				}
				$this->persistenceManager->persistAll();
		}

		} catch (\Exception $e) {
			$this->response->setStatus(500, "Internal Server Error");
		}
		return "";

	}

	/**
	 * get Details of users (GET HTTP method)
	 * @return void
	 */
	public function getUserAction() {
		$user    = array();
		$id = NULL;
		try {
			$checkToken = $this->hasAuthenticatedUser();
			if( $checkToken === true ) {
				if ($this->request->hasArgument("id") == true) {
					$id = $this->request->getArgument("id");
				}

				if (empty($id) == false) {
					$users = $this->userRepository->findOneById($id);
				} else {
					$users = $this->userRepository->findAll();
				}

				if (empty($users) == true) {
					$this->response->setStatus(204, "No Content");
				} else {
					$i = 0;
					if ($users instanceof \Joson\Rest\Domain\Model\User) {
						$user[$i]["id"] = $users->getId();
						$user[$i]["name"] = $users->getName();
						$user[$i]["age"]  = $users->getAge();
					} else {
						foreach ($users as $userDetails) {
							$user[$i]["id"] = $userDetails->getId();
							$user[$i]["name"] = $userDetails->getName();
							$user[$i]["age"]  = $userDetails->getAge();
							$i++;
						}
					}

				}
			}

			$this->persistenceManager->persistAll();
		} catch (\Exception $e) {
			$this->response->setStatus(500, "Internal Server Error");
		}

		return \json_encode($user);

	}

	/**
	 * Remove user from user table (DELETE HTTP method)
	 * @return void
	 */
	public function removeUserAction() {
		try {
			$checkToken = $this->hasAuthenticatedUser();
			if( $checkToken == true ) {
				if ($this->request->hasArgument("id") === true) {
					$id    = $this->request->getArgument("id");
					$users = $this->userRepository->findOneById($id);
					if (empty($users) === true) {
						$this->response->setStatus(204, "No Content");
					} else {
						$this->userRepository->remove($users);
					}
				} else {
					$this->response->setStatus(400, "Bad Request");
				}
				$this->persistenceManager->persistAll();
			}

		} catch (\Exception $e) {
			$this->response->setStatus(500, "Internal Server Error");
		}
		return "";
	}

	/**
	 * Update the user details (POST HTTP method)
	 * @return void
	 */
	public function updateUserAction() {

		$flag1   = 0;
		$id   = 0;
		$name = "";
		$age  = 0;

		try {
			$checkToken = $this->hasAuthenticatedUser();
			if( $checkToken === true ) {
				if ($this->request->hasArgument("id") === false) {
					$this->response->setStatus(400, "Bad Request");
					$flag1   = 1;
				} else {
					$id = $this->request->getArgument("id");
					if (empty($id) === true) {
						$this->response->setStatus(400, "Bad Request");
						$flag1   = 1;
					} else {
						if ($this->request->hasArgument("name") === false) {
							$this->response->setStatus(400, "Bad Request");
							$flag1   = 1;
						} else {
							$name = $this->request->getArgument("name");
							if (empty($name) === true) {
								$this->response->setStatus(400, "Bad Request");
								$flag1   = 1;
							} else {
								if ($this->request->hasArgument("age") === false) {
									$this->response->setStatus(400, "Bad Request");
									$flag1   = 1;
								} else {
									$age = $this->request->getArgument("age");
									if (empty($age) === true) {
										$this->response->setStatus(400, "Bad Request");
										$flag1   = 1;
									}
								}
							}
						}
					}
				}

				if ($flag1 === 0) {
					$users = $this->userRepository->findOneById($id);
					if (empty($users) === true) {
						$this->response->setStatus(204, "No Content");
					} else {
						$users->setName($name);
						$users->setAge($age);
						$this->userRepository->update($users);
					}
				}
			}
			$this->persistenceManager->persistAll();
		} catch (\Exception $e) {
			$this->response->setStatus(500, "Internal Server Error");
		}

		return "";
	}

	/**
	* Update small part of user details (PATCH HTTP method)
	* @return void
	*/
	public function updatePartialUserDetailsAction() {
		$flag1   = 0;
		$id   = 0;
		$name = "";
		$age  = 0;
		try {
			$checkToken = $this->hasAuthenticatedUser();
			if( $checkToken === true ) {
				if( $this->request->hasArgument("id") === false ) {
					$this->response->setStatus(400, "Bad Request");
					$flag1   = 1;
				} else {
					$id = $this->request->getArgument("id");
					if( empty($id) === true ) {
						$this->response->setStatus(400, "Bad Request");
						$flag1   = 1;
					}
				}

				if($flag1 === 0) {
					$user = $this->userRepository->findOneById($id);
					if( empty($user) === true ) {
						$this->response->setStatus(204, "No Content");
					} else {
						if( $this->request->hasArgument("name") === true ) {
							$name = $this->request->getArgument("name");
							if( empty($name) === true ) {
								$name = $user->getName();
							}
						} else {
							$name = $user->getName();
						}

						if( $this->request->hasArgument("age") === true ) {
							$age = $this->request->getArgument("age");
							if( empty($age) === true ) {
								$age = $user->getAge();
							}
						} else {
							$age = $user->getAge();
						}
						$user->setName($name);
						$user->setAge($age);
						$this->userRepository->update($user);
					}

				}

			}
			$this->persistenceManager->persistAll();

		} catch (\Exception $e) {
			$this->response->setStatus(500, "Internal Server Error");
		}
		return "";

	}

	/**
	* Get the list of available http methods for this restful service (OPTIONS HTTP method)
	* @return void
	*/
	public function getAvailableHttpMethodsAction() {
		$checkToken = $this->hasAuthenticatedUser();
		if( $checkToken === true ) {
			$options = array(
				"Allow"=>"PUT,GET,DELETE,POST,PATCH,OPTIONS"
			);
			return \json_encode($options);
		}
		return "";
	}

	/**
	* Check whether authenticated token is valid or not
	* @return boolean
	*/
	private function hasAuthenticatedUser() {
		$currentHeaders = $this->response->getHeaders();
		$currentHeaders->set('Content-Type', 'text/json; charset=utf-8');
		$this->response->sendHeaders();
		$returnResult = true;

		// Authenticating user
		$httpRequestHeaders = $this->request->getHttpRequest()->getHeaders();
		if( empty($httpRequestHeaders) === true ) {
			$this->response->setStatus(400, "Bad Request");
			$returnResult = false;
		} else {
			if( $httpRequestHeaders->has("Token") === false ) {
				$this->response->setStatus(400, "Bad Request");
				$returnResult = false;
			} else {
				$authToken = $httpRequestHeaders->get("Token");
				if( empty($authToken) ===  true ) {
					$this->response->setStatus(401, "Unauthorized");
					$returnResult = false;
			} else {
					$authUser = $this->realUsersRepository->findOneByToken($authToken);
					if( empty($authUser) === true ) {
						$this->response->setStatus(401, "Unauthorized");
						$returnResult = false;
					}
				}
			}
		}
		return $returnResult;
	}

}
